/*
 * Maintain registry of classes. The state is stored in the
 * interpreter (as Tcl variables), so we can register classes
 * in each interpreter separately.
 */

#include <tcl.h>
#include <stdlib.h>
#include "tclobj.h"
#include "register.h"
#include "compat.h"
#include "signature.h"

/*
 * cvs version
 */
char * tclobj_register_id = "$Id: register.cc,v 1.2 1996/11/28 15:12:11 fp Exp $";


/*
 * next available identifier. we cannot store this in the interpreter,
 * because we expect the identifiers to be unique over interpreter
 * boundaries.
 */

static unsigned long TclObj_NextFreeIdentifier = 256;

/*
 * register new class with the interpreter, initializing it if
 * necessary
 */

unsigned long
TclObj_RegisterClass (Tcl_Interp *interp, TclObj_Info *theinfo)
{
  char tmpstr[32], str2[32], *ptr;
  unsigned long regcount;
  int needtoinitialize=0;

  if (theinfo->ident == 0) {
    theinfo->counter = 0;
    theinfo->ident   = TclObj_NextFreeIdentifier++;
    needtoinitialize = 1;
  }

  sprintf     (tmpstr, "%lu", (unsigned long) theinfo);
  Tcl_SetVar2 (interp, "TclObj_ClassInfo", (char *) theinfo->name,
	       tmpstr, TCL_GLOBAL_ONLY);

  sprintf     (str2,   "%lu", (unsigned long) theinfo->ident);
  Tcl_SetVar2 (interp, "TclObj_ClassInfo", str2, tmpstr,
	       TCL_GLOBAL_ONLY);

  if (needtoinitialize) {
    TclObj_BuildComptree (interp, theinfo);
  }

  return theinfo->ident;
}

/*
 * Query info
 */

TclObj_Info *
TclObj_GetClassInfo (Tcl_Interp *interp, unsigned long ident)
{
  char *ptrname, tmpstr[32];
  
  sprintf (tmpstr, "%lu", (unsigned long) ident);
  if ((ptrname = Tcl_GetVar2 (interp, "TclObj_ClassInfo", tmpstr,
			      TCL_GLOBAL_ONLY)) == NULL)
    return NULL;
  
  return (TclObj_Info*) strtoul (ptrname, NULL, 0);
}

TclObj_Info *
TclObj_GetClassInfo (Tcl_Interp *interp, const char *thetype)
{
  char *ptrname;
  
  if ((ptrname = Tcl_GetVar2 (interp, "TclObj_ClassInfo", 
			      (char *) thetype,
			      TCL_GLOBAL_ONLY)) == NULL)
    return NULL;

  return (TclObj_Info*) strtoul (ptrname, NULL, 0);
}
